#
# Aura Engine (https://gitlab.servus.at/aura/engine)
#
# Copyright (C) 2022 - The Aura Engine Team.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging

from src.base.config import AuraConfig
from src.base.api import SimpleApi
from src.resources import ResourceUtil


class ClockInfoHandler:
    """
    Sends current studio clock information to the `engine-api` REST endpoint,
    whenever a new play command is issued.
    """

    logger = None
    config = None
    api = None
    engine = None

    def __init__(self, engine):
        """
        Initialize.
        """
        self.logger = logging.getLogger("AuraEngine")
        self.config = AuraConfig.config()
        self.api = SimpleApi()
        self.engine = engine

    def on_play(self, _):
        """
        Event Handler which is called by the engine when some play command to Liquidsoap is issued.
        This does not indicate that Liquidsoap started playing actually, only that the command has
        been issued. Naturally this happens slightly before single tracks play audio.

        This event is not issued when media is played by Liquidsoap in fallback scenarios.

        Args:
            entry (PlaylistEntry):
        """
        if not self.engine:
            self.logger.critical("ClockInfoHandler | Engine instance not available!")
            return

        active_playlist_type, active_playlist = self.engine.scheduler.get_active_playlist()
        active_type_id = active_playlist_type.get("id")
        upcoming_timeslots = self.engine.scheduler.get_upcoming_timeslots()
        self.post_clock_info(active_type_id, active_playlist, upcoming_timeslots)

    def post_clock_info(self, active_type, active_playlist, upcoming_timeslots):
        """
        Posts current information on timeslots and playlist to the Engine API clock endpoint.
        """
        active_timeslot = None
        next_timeslot = None

        if len(upcoming_timeslots) >= 1:
            active_timeslot = upcoming_timeslots[0]
        if len(upcoming_timeslots) >= 2:
            next_timeslot = upcoming_timeslots[1]

        data = {
            "engine_source": self.config.get("api_engine_number"),
            "current_timeslot": self.build_timeslot(active_timeslot, active_type),
            "next_timeslot": self.build_timeslot(next_timeslot),
            "planned_playlist": self.build_playlist(active_playlist),
        }
        url = self.config.get("api_engine_store_clock")
        self.logger.info(f"PUT clock info to '{url}': \n{data}")
        self.api.put(url, data=data)

    def build_timeslot(self, timeslot, playlist_type: int = 0) -> dict:
        """
        Transforms a `Timeslot` object to a dictionary digestable by the clock API endpoint.

        Args:
            timeslot (Timeslot)
            playlist_type (int): Indicates if the playlist was assigned on
                (0) timeslot level (default)
                (1) schedule level
                (2) show level
                (3) station level
        """
        if not timeslot:
            return

        # #FIXME fallback_type should be renamed to something like "playlist_type" in API
        return {
            "timeslot_id": timeslot.timeslot_id,
            "timeslot_start": timeslot.timeslot_start,
            "timeslot_end": timeslot.timeslot_end,
            "show_id": timeslot.show_id,
            "show_name": timeslot.show_name,
            "playlist_id": timeslot.playlist_id,
            "fallback_type": playlist_type,
        }

    def build_playlist(self, playlist) -> dict:
        """
        Transforms a `Playlist` object to a dictionary digestable by the clock API endpoint.
        """
        if not playlist:
            return None

        entries = []
        for e in playlist.entries:
            content_class = ResourceUtil.get_content_class(e.get_content_type())
            entry = {
                "track_type": int(content_class.numeric),
                "track_start": e.entry_start,
                "track_num": e.entry_num,
                "track_duration": e.duration,
            }
            if e.meta_data:
                entry |= {
                    "track_artist": e.meta_data.artist,
                    "track_album": e.meta_data.album,
                    "track_title": e.meta_data.title,
                }
            entries.append(entry)

        pls_dict = {"playlist_id": playlist.playlist_id, "entries": entries}
        return pls_dict
